package com.sun.org.apache.xalan.internal.xsltc.trax;

import com.sun.org.apache.xalan.internal.xsltc.runtime.MessageHandler;
import com.sun.org.apache.xml.internal.utils.SystemIDResolver;
import java.util.HashMap;
import com.sun.org.apache.xml.internal.serializer.OutputPropertiesFactory;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Enumeration;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import com.sun.org.apache.xalan.internal.xsltc.TransletException;
import jdk.xml.internal.JdkXmlUtils;
import java.io.Reader;
import java.io.InputStream;
import org.xml.sax.XMLReader;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamReader;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import com.sun.org.apache.xalan.internal.xsltc.dom.SAXImpl;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stax.StAXSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;
import com.sun.org.apache.xml.internal.dtm.DTMWSFilter;
import com.sun.org.apache.xalan.internal.xsltc.dom.DOMWSFilter;
import com.sun.org.apache.xalan.internal.xsltc.StripFilter;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.ContentHandler;
import java.net.URLConnection;
import java.io.Writer;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import java.net.UnknownServiceException;
import java.net.URI;
import java.net.URL;
import java.io.FileOutputStream;
import java.io.File;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.sax.SAXResult;
import com.sun.org.apache.xml.internal.serializer.SerializationHandler;
import javax.xml.transform.stax.StAXResult;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.TransformerException;
import com.sun.org.apache.xalan.internal.xsltc.compiler.util.ErrorMsg;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import com.sun.org.apache.xalan.internal.xsltc.Translet;
import java.util.Map;
import com.sun.org.apache.xalan.internal.utils.XMLSecurityManager;
import com.sun.org.apache.xml.internal.utils.XMLReaderManager;
import com.sun.org.apache.xalan.internal.xsltc.dom.XSLTCDTMManager;
import java.io.OutputStream;
import com.sun.org.apache.xalan.internal.xsltc.DOM;
import com.sun.org.apache.xalan.internal.xsltc.runtime.output.TransletOutputHandlerFactory;
import java.util.Properties;
import javax.xml.transform.URIResolver;
import com.sun.org.apache.xalan.internal.xsltc.runtime.AbstractTranslet;
import javax.xml.transform.ErrorListener;
import com.sun.org.apache.xalan.internal.xsltc.DOMCache;
import javax.xml.transform.Transformer;

public final class TransformerImpl extends Transformer implements DOMCache, ErrorListener
{
    private static final String LEXICAL_HANDLER_PROPERTY = "http://xml.org/sax/properties/lexical-handler";
    private static final String NAMESPACE_PREFIXES_FEATURE = "http://xml.org/sax/features/namespace-prefixes";
    private AbstractTranslet _translet;
    private String _method;
    private String _encoding;
    private String _sourceSystemId;
    private ErrorListener _errorListener;
    private URIResolver _uriResolver;
    private Properties _properties;
    private Properties _propertiesClone;
    private TransletOutputHandlerFactory _tohFactory;
    private DOM _dom;
    private int _indentNumber;
    private TransformerFactoryImpl _tfactory;
    private OutputStream _ostream;
    private XSLTCDTMManager _dtmManager;
    private XMLReaderManager _readerManager;
    private boolean _isIdentity;
    private boolean _isSecureProcessing;
    private boolean _overrideDefaultParser;
    private String _accessExternalDTD;
    private XMLSecurityManager _securityManager;
    private Map<String, Object> _parameters;
    
    protected TransformerImpl(Properties outputProperties, int indentNumber, TransformerFactoryImpl tfactory) {
        this(null, outputProperties, indentNumber, tfactory);
        this._isIdentity = true;
    }
    
    protected TransformerImpl(Translet translet, Properties outputProperties, int indentNumber, TransformerFactoryImpl tfactory) {
        this._translet = null;
        this._method = null;
        this._encoding = null;
        this._sourceSystemId = null;
        this._errorListener = this;
        this._uriResolver = null;
        this._tohFactory = null;
        this._dom = null;
        this._tfactory = null;
        this._ostream = null;
        this._dtmManager = null;
        this._isIdentity = false;
        this._isSecureProcessing = false;
        this._accessExternalDTD = "all";
        this._parameters = null;
        this._translet = (AbstractTranslet)translet;
        this._properties = this.createOutputProperties(outputProperties);
        this._propertiesClone = (Properties)this._properties.clone();
        this._indentNumber = indentNumber;
        this._tfactory = tfactory;
        this._overrideDefaultParser = this._tfactory.overrideDefaultParser();
        this._accessExternalDTD = (String)this._tfactory.getAttribute("http://javax.xml.XMLConstants/property/accessExternalDTD");
        this._securityManager = (XMLSecurityManager)this._tfactory.getAttribute("http://apache.org/xml/properties/security-manager");
        (this._readerManager = XMLReaderManager.getInstance(this._overrideDefaultParser)).setProperty("http://javax.xml.XMLConstants/property/accessExternalDTD", this._accessExternalDTD);
        this._readerManager.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", this._isSecureProcessing);
        this._readerManager.setProperty("http://apache.org/xml/properties/security-manager", this._securityManager);
    }
    
    public boolean isSecureProcessing() {
        return this._isSecureProcessing;
    }
    
    public void setSecureProcessing(boolean flag) {
        this._isSecureProcessing = flag;
        this._readerManager.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", this._isSecureProcessing);
    }
    
    public boolean overrideDefaultParser() {
        return this._overrideDefaultParser;
    }
    
    public void setOverrideDefaultParser(boolean flag) {
        this._overrideDefaultParser = flag;
    }
    
    protected AbstractTranslet getTranslet() {
        return this._translet;
    }
    
    public boolean isIdentity() {
        return this._isIdentity;
    }
    
    @Override
    public void transform(Source source, Result result) throws TransformerException {
        if (!this._isIdentity) {
            if (this._translet == null) {
                ErrorMsg err = new ErrorMsg("JAXP_NO_TRANSLET_ERR");
                throw new TransformerException(err.toString());
            }
            this.transferOutputProperties(this._translet);
        }
        SerializationHandler toHandler = this.getOutputHandler(result);
        if (toHandler == null) {
            ErrorMsg err2 = new ErrorMsg("JAXP_NO_HANDLER_ERR");
            throw new TransformerException(err2.toString());
        }
        if (this._uriResolver != null && !this._isIdentity) {
            this._translet.setDOMCache(this);
        }
        if (this._isIdentity) {
            this.transferOutputProperties(toHandler);
        }
        this.transform(source, toHandler, this._encoding);
        try {
            if ((result instanceof DOMResult)) {
                ((DOMResult)result).setNode(this._tohFactory.getNode());
            }
            else if ((result instanceof StAXResult)) {
                if (((StAXResult)result).getXMLEventWriter() != null) {
                    this._tohFactory.getXMLEventWriter().flush();
                }
                else if (((StAXResult)result).getXMLStreamWriter() != null) {
                    this._tohFactory.getXMLStreamWriter().flush();
                }
            }
        }
        catch (Exception e) {
            System.out.println("Result writing error");
        }
        return;
    }
    
    public SerializationHandler getOutputHandler(Result result) throws TransformerException {
        this._method = (String)this._properties.get("method");
        this._encoding = this._properties.getProperty("encoding");
        (this._tohFactory = TransletOutputHandlerFactory.newInstance(this._overrideDefaultParser)).setEncoding(this._encoding);
        if (this._method != null) {
            this._tohFactory.setOutputMethod(this._method);
        }
        if (this._indentNumber >= 0) {
            this._tohFactory.setIndentNumber(this._indentNumber);
        }
        try {
            if ((result instanceof SAXResult)) {
                SAXResult target2 = (SAXResult)result;
                ContentHandler handler = target2.getHandler();
                this._tohFactory.setHandler(handler);
                LexicalHandler lexicalHandler = target2.getLexicalHandler();
                if (lexicalHandler != null) {
                    this._tohFactory.setLexicalHandler(lexicalHandler);
                }
                this._tohFactory.setOutputType(1);
                return this._tohFactory.getSerializationHandler();
            }
            else if ((result instanceof StAXResult)) {
                if (((StAXResult)result).getXMLEventWriter() != null) {
                    this._tohFactory.setXMLEventWriter(((StAXResult)result).getXMLEventWriter());
                }
                else if (((StAXResult)result).getXMLStreamWriter() != null) {
                    this._tohFactory.setXMLStreamWriter(((StAXResult)result).getXMLStreamWriter());
                }
                this._tohFactory.setOutputType(3);
                return this._tohFactory.getSerializationHandler();
            }
            else {
                if ((result instanceof DOMResult)) {
                    this._tohFactory.setNode(((DOMResult)result).getNode());
                    this._tohFactory.setNextSibling(((DOMResult)result).getNextSibling());
                    this._tohFactory.setOutputType(2);
                    return this._tohFactory.getSerializationHandler();
                }
                if ((result instanceof StreamResult)) {
                    StreamResult target = (StreamResult)result;
                    this._tohFactory.setOutputType(0);
                    Writer writer = target.getWriter();
                    if (writer != null) {
                        this._tohFactory.setWriter(writer);
                        return this._tohFactory.getSerializationHandler();
                    }
                    OutputStream ostream = target.getOutputStream();
                    if (ostream != null) {
                        this._tohFactory.setOutputStream(ostream);
                        return this._tohFactory.getSerializationHandler();
                    }
                    String systemId = result.getSystemId();
                    if (systemId == null) {
                        ErrorMsg err = new ErrorMsg("JAXP_NO_RESULT_ERR");
                        throw new TransformerException(err.toString());
                    }
                    if (systemId.startsWith("file:")) {
                        try {
                            URI uri = new URI(systemId);
                            systemId = "file:";
                            String host = uri.getHost();
                            String path = uri.getPath();
                            if (path == null) {
                                path = "";
                            }
                            if (host != null) {
                                systemId = systemId + "//" + host + path;
                            }
                            else {
                                systemId = systemId + "//" + path;
                            }
                        }
                        catch (Exception ex) {}
                        URL url = new URL(systemId);
                        this._ostream = new FileOutputStream(url.getFile());
                        this._tohFactory.setOutputStream(this._ostream);
                        return this._tohFactory.getSerializationHandler();
                    }
                    if (systemId.startsWith("http:")) {
                        URL url = new URL(systemId);
                        URLConnection connection = url.openConnection();
                        this._tohFactory.setOutputStream(this._ostream = connection.getOutputStream());
                        return this._tohFactory.getSerializationHandler();
                    }
                    this._tohFactory.setOutputStream(this._ostream = new FileOutputStream(new File(systemId)));
                    return this._tohFactory.getSerializationHandler();
                }
            }
        }
        catch (UnknownServiceException e) {
            throw new TransformerException(e);
        }
        catch (ParserConfigurationException e2) {
            throw new TransformerException(e2);
        }
        catch (IOException e3) {
            throw new TransformerException(e3);
        }
        return null;
    }
    
    protected void setDOM(DOM dom) {
        this._dom = dom;
    }
    
    private DOM getDOM(Source source) throws TransformerException {
        try {
            DOM dom;
            if (source != null) {
                DTMWSFilter wsfilter;
                if (this._translet != null && this._translet instanceof StripFilter) {
                    wsfilter = new DOMWSFilter(this._translet);
                }
                else {
                    wsfilter = null;
                }
                boolean hasIdCall = this._translet != null && this._translet.hasIdCall();
                if (this._dtmManager == null) {
                    (this._dtmManager = this._tfactory.createNewDTMManagerInstance()).setOverrideDefaultParser(this._overrideDefaultParser);
                }
                dom = (DOM)this._dtmManager.getDTM(source, false, wsfilter, true, false, false, 0, hasIdCall);
            }
            else if (this._dom != null) {
                dom = this._dom;
                this._dom = null;
            }
            else {
                return null;
            }
            if (!this._isIdentity) {
                this._translet.prepassDocument(dom);
                return dom;
            }
            return dom;
        }
        catch (Exception e) {
            if (this._errorListener != null) {
                this.postErrorToListener(e.getMessage());
            }
            throw new TransformerException(e);
        }
    }
    
    protected TransformerFactoryImpl getTransformerFactory() {
        return this._tfactory;
    }
    
    protected TransletOutputHandlerFactory getTransletOutputHandlerFactory() {
        return this._tohFactory;
    }
    
    private void transformIdentity(Source source, SerializationHandler handler) throws Exception {
        if (source != null) {
            this._sourceSystemId = source.getSystemId();
        }
        if ((source instanceof StreamSource)) {
            StreamSource stream = (StreamSource)source;
            InputStream streamInput = stream.getInputStream();
            Reader streamReader = stream.getReader();
            XMLReader reader2 = this._readerManager.getXMLReader();
            try {
                try {
                    reader2.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
                    reader2.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
                }
                catch (SAXException ex2) {}
                reader2.setContentHandler(handler);
                InputSource input2;
                if (streamInput != null) {
                    input2 = new InputSource(streamInput);
                    input2.setSystemId(this._sourceSystemId);
                }
                else if (streamReader != null) {
                    input2 = new InputSource(streamReader);
                    input2.setSystemId(this._sourceSystemId);
                }
                else if (this._sourceSystemId != null) {
                    input2 = new InputSource(this._sourceSystemId);
                }
                else {
                    ErrorMsg err2 = new ErrorMsg("JAXP_NO_SOURCE_ERR");
                    throw new TransformerException(err2.toString());
                }
                reader2.parse(input2);
            }
            finally {
                this._readerManager.releaseXMLReader(reader2);
            }
        }
        else if ((source instanceof SAXSource)) {
            SAXSource sax = (SAXSource)source;
            XMLReader reader = sax.getXMLReader();
            InputSource input = sax.getInputSource();
            boolean userReader = true;
            try {
                if (reader == null) {
                    reader = this._readerManager.getXMLReader();
                    userReader = false;
                }
                try {
                    reader.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
                    reader.setFeature("http://xml.org/sax/features/namespace-prefixes", true);
                }
                catch (SAXException ex) {}
                reader.setContentHandler(handler);
                reader.parse(input);
            }
            finally {
                if (!userReader) {
                    this._readerManager.releaseXMLReader(reader);
                }
            }
        }
        else if ((source instanceof StAXSource)) {
            StAXSource staxSource = (StAXSource)source;
            if (staxSource.getXMLEventReader() != null) {
                XMLEventReader xmlEventReader = staxSource.getXMLEventReader();
                StAXEvent2SAX staxevent2sax = new StAXEvent2SAX(xmlEventReader);
                staxevent2sax.setContentHandler(handler);
                staxevent2sax.parse();
                handler.flushPending();
            }
            else if (staxSource.getXMLStreamReader() != null) {
                XMLStreamReader xmlStreamReader = staxSource.getXMLStreamReader();
                StAXStream2SAX staxStream2SAX = new StAXStream2SAX(xmlStreamReader);
                staxStream2SAX.setContentHandler(handler);
                staxStream2SAX.parse();
                handler.flushPending();
            }
        }
        else if ((source instanceof DOMSource)) {
            DOMSource domsrc = (DOMSource)source;
            new DOM2TO(domsrc.getNode(), handler).parse();
        }
        else if ((source instanceof XSLTCSource)) {
            DOM dom = ((XSLTCSource)source).getDOM(null, this._translet);
            ((SAXImpl)dom).copy(handler);
        }
        else {
            ErrorMsg err = new ErrorMsg("JAXP_NO_SOURCE_ERR");
            throw new TransformerException(err.toString());
        }
    }
    
    private void transform(Source source, SerializationHandler handler, String encoding) throws TransformerException {
        try {
            if ((source instanceof StreamSource && source.getSystemId() == null && ((StreamSource)source).getInputStream() == null && ((StreamSource)source).getReader() == null) || (source instanceof SAXSource && ((SAXSource)source).getInputSource() == null && ((SAXSource)source).getXMLReader() == null) || (source instanceof DOMSource && ((DOMSource)source).getNode() == null)) {
                DocumentBuilderFactory builderF = JdkXmlUtils.getDOMFactory(this._overrideDefaultParser);
                DocumentBuilder builder = builderF.newDocumentBuilder();
                String systemID = source.getSystemId();
                source = new DOMSource(builder.newDocument());
                if (systemID != null) {
                    source.setSystemId(systemID);
                }
            }
            if (this._isIdentity) {
                this.transformIdentity(source, handler);
            }
            else {
                this._translet.transform(this.getDOM(source), handler);
            }
        }
        catch (TransletException e) {
            if (this._errorListener != null) {
                this.postErrorToListener(e.getMessage());
            }
            throw new TransformerException(e);
        }
        catch (RuntimeException e2) {
            if (this._errorListener != null) {
                this.postErrorToListener(e2.getMessage());
            }
            throw new TransformerException(e2);
        }
        catch (Exception e3) {
            if (this._errorListener != null) {
                this.postErrorToListener(e3.getMessage());
            }
            throw new TransformerException(e3);
        }
        finally {
            this._dtmManager = null;
        }
        if (this._ostream != null) {
            try {
                this._ostream.close();
            }
            catch (IOException ex) {}
            this._ostream = null;
        }
    }
    
    @Override
    public ErrorListener getErrorListener() {
        return this._errorListener;
    }
    
    @Override
    public void setErrorListener(ErrorListener listener) throws IllegalArgumentException {
        if (listener == null) {
            ErrorMsg err = new ErrorMsg("ERROR_LISTENER_NULL_ERR", "Transformer");
            throw new IllegalArgumentException(err.toString());
        }
        this._errorListener = listener;
        if (this._translet != null) {
            this._translet.setMessageHandler(new TransformerImpl.MessageHandler(this._errorListener));
        }
        return;
    }
    
    private void postErrorToListener(String message) {
        try {
            this._errorListener.error(new TransformerException(message));
        }
        catch (TransformerException ex) {}
    }
    
    private void postWarningToListener(String message) {
        try {
            this._errorListener.warning(new TransformerException(message));
        }
        catch (TransformerException ex) {}
    }
    
    @Override
    public Properties getOutputProperties() {
        return (Properties)this._properties.clone();
    }
    
    @Override
    public String getOutputProperty(String name) throws IllegalArgumentException {
        if (!this.validOutputProperty(name)) {
            ErrorMsg err = new ErrorMsg("JAXP_UNKNOWN_PROP_ERR", name);
            throw new IllegalArgumentException(err.toString());
        }
        return this._properties.getProperty(name);
    }
    
    @Override
    public void setOutputProperties(Properties properties) throws IllegalArgumentException {
        if (properties != null) {
            Enumeration<Object> names = properties.propertyNames();
            while (names.hasMoreElements()) {
                String name = (String)names.nextElement();
                if (this.isDefaultProperty(name, properties)) {
                    continue;
                }
                if (this.validOutputProperty(name)) {
                    this._properties.setProperty(name, properties.getProperty(name));
                }
                else {
                    ErrorMsg err = new ErrorMsg("JAXP_UNKNOWN_PROP_ERR", name);
                    throw new IllegalArgumentException(err.toString());
                }
            }
        }
        else {
            this._properties = this._propertiesClone;
        }
    }
    
    @Override
    public void setOutputProperty(String name, String value) throws IllegalArgumentException {
        if (!this.validOutputProperty(name)) {
            ErrorMsg err = new ErrorMsg("JAXP_UNKNOWN_PROP_ERR", name);
            throw new IllegalArgumentException(err.toString());
        }
        this._properties.setProperty(name, value);
        return;
    }
    
    private void transferOutputProperties(AbstractTranslet translet) {
        if (this._properties == null) {
            return;
        }
        Enumeration<Object> names = this._properties.propertyNames();
        while (names.hasMoreElements()) {
            String name = (String)names.nextElement();
            String value = (String)this._properties.get(name);
            if (value == null) {
                continue;
            }
            if (name.equals("encoding")) {
                translet._encoding = value;
            }
            else if (name.equals("method")) {
                translet._method = value;
            }
            else if (name.equals("doctype-public")) {
                translet._doctypePublic = value;
            }
            else if (name.equals("doctype-system")) {
                translet._doctypeSystem = value;
            }
            else if (name.equals("media-type")) {
                translet._mediaType = value;
            }
            else if (name.equals("standalone")) {
                translet._standalone = value;
            }
            else if (name.equals("version")) {
                translet._version = value;
            }
            else if (name.equals("omit-xml-declaration")) {
                translet._omitHeader = (value != null && value.toLowerCase().equals("yes"));
            }
            else if (name.equals("indent")) {
                translet._indent = (value != null && value.toLowerCase().equals("yes"));
            }
            else if (name.equals("{http://xml.apache.org/xslt}indent-amount")) {
                if (value != null) {
                    translet._indentamount = Integer.parseInt(value);
                }
                else {
                    continue;
                }
            }
            else if (name.equals("{http://xml.apache.org/xalan}indent-amount")) {
                if (value != null) {
                    translet._indentamount = Integer.parseInt(value);
                }
                else {
                    continue;
                }
            }
            else if (name.equals("cdata-section-elements")) {
                if (value != null) {
                    translet._cdata = null;
                    StringTokenizer e = new StringTokenizer(value);
                    while (e.hasMoreTokens()) {
                        translet.addCdataElement(e.nextToken());
                    }
                }
                else {
                    continue;
                }
            }
            else if (name.equals("http://www.oracle.com/xml/is-standalone") && value != null && value.equals("yes")) {
                translet._isStandalone = true;
            }
            else {
                continue;
            }
        }
        return;
    }
    
    public void transferOutputProperties(SerializationHandler handler) {
        if (this._properties == null) {
            return;
        }
        String doctypePublic = null;
        String doctypeSystem = null;
        Enumeration<Object> names = this._properties.propertyNames();
        while (names.hasMoreElements()) {
            String name = (String)names.nextElement();
            String value = (String)this._properties.get(name);
            if (value == null) {
                continue;
            }
            if (name.equals("doctype-public")) {
                doctypePublic = value;
            }
            else if (name.equals("doctype-system")) {
                doctypeSystem = value;
            }
            else if (name.equals("media-type")) {
                handler.setMediaType(value);
            }
            else if (name.equals("standalone")) {
                handler.setStandalone(value);
            }
            else if (name.equals("version")) {
                handler.setVersion(value);
            }
            else if (name.equals("omit-xml-declaration")) {
                handler.setOmitXMLDeclaration(value != null && value.toLowerCase().equals("yes"));
            }
            else if (name.equals("indent")) {
                handler.setIndent(value != null && value.toLowerCase().equals("yes"));
            }
            else if (name.equals("{http://xml.apache.org/xslt}indent-amount")) {
                if (value != null) {
                    handler.setIndentAmount(Integer.parseInt(value));
                }
                else {
                    continue;
                }
            }
            else if (name.equals("{http://xml.apache.org/xalan}indent-amount")) {
                if (value != null) {
                    handler.setIndentAmount(Integer.parseInt(value));
                }
                else {
                    continue;
                }
            }
            else if (name.equals("http://www.oracle.com/xml/is-standalone")) {
                if (value != null && value.equals("yes")) {
                    handler.setIsStandalone(true);
                }
                else {
                    continue;
                }
            }
            else if (name.equals("cdata-section-elements") && value != null) {
                StringTokenizer e = new StringTokenizer(value);
                ArrayList<String> uriAndLocalNames = null;
                while (e.hasMoreTokens()) {
                    String token = e.nextToken();
                    int lastcolon = token.lastIndexOf(58);
                    String uri;
                    String localName;
                    if (lastcolon > 0) {
                        uri = token.substring(0, lastcolon);
                        localName = token.substring(lastcolon + 1);
                    }
                    else {
                        uri = null;
                        localName = token;
                    }
                    if (uriAndLocalNames == null) {
                        uriAndLocalNames = new ArrayList();
                    }
                    uriAndLocalNames.add(uri);
                    uriAndLocalNames.add(localName);
                }
                handler.setCdataSectionElements(uriAndLocalNames);
            }
            else {
                continue;
            }
        }
        if (doctypePublic != null || doctypeSystem != null) {
            handler.setDoctype(doctypeSystem, doctypePublic);
        }
        return;
    }
    
    private Properties createOutputProperties(Properties outputProperties) {
        Properties defaults = new Properties();
        this.setDefaults(defaults, "xml");
        Properties base = new Properties(defaults);
        if (outputProperties != null) {
            Enumeration<Object> names = outputProperties.propertyNames();
            while (names.hasMoreElements()) {
                String name = (String)names.nextElement();
                base.setProperty(name, outputProperties.getProperty(name));
            }
        }
        else {
            base.setProperty("encoding", this._translet._encoding);
            if (this._translet._method != null) {
                base.setProperty("method", this._translet._method);
            }
        }
        String method = base.getProperty("method");
        if (method != null) {
            if (method.equals("html")) {
                this.setDefaults(defaults, "html");
            }
            else if (method.equals("text")) {
                this.setDefaults(defaults, "text");
            }
        }
        return base;
    }
    
    private void setDefaults(Properties props, String method) {
        Properties method_props = OutputPropertiesFactory.getDefaultMethodProperties(method);
        Enumeration<Object> names = method_props.propertyNames();
        while (names.hasMoreElements()) {
            String name = (String)names.nextElement();
            props.setProperty(name, method_props.getProperty(name));
        }
    }
    
    private boolean validOutputProperty(String name) {
        return name.equals("encoding") || name.equals("method") || name.equals("indent") || name.equals("doctype-public") || name.equals("doctype-system") || name.equals("cdata-section-elements") || name.equals("media-type") || name.equals("omit-xml-declaration") || name.equals("standalone") || name.equals("version") || name.equals("http://www.oracle.com/xml/is-standalone") || name.charAt(0) == '{';
    }
    
    private boolean isDefaultProperty(String name, Properties properties) {
        return properties.get(name) == null;
    }
    
    @Override
    public void setParameter(String name, Object value) {
        if (value == null) {
            ErrorMsg err = new ErrorMsg("JAXP_INVALID_SET_PARAM_VALUE", name);
            throw new IllegalArgumentException(err.toString());
        }
        if (this._isIdentity) {
            if (this._parameters == null) {
                this._parameters = new HashMap();
            }
            this._parameters.put(name, value);
        }
        else {
            this._translet.addParameter(name, value);
        }
        return;
    }
    
    @Override
    public void clearParameters() {
        if (this._isIdentity && this._parameters != null) {
            this._parameters.clear();
        }
        else {
            this._translet.clearParameters();
        }
    }
    
    @Override
    public final Object getParameter(String name) {
        if (this._isIdentity) {
            return (this._parameters != null) ? this._parameters.get(name) : null;
        }
        return this._translet.getParameter(name);
    }
    
    @Override
    public URIResolver getURIResolver() {
        return this._uriResolver;
    }
    
    @Override
    public void setURIResolver(URIResolver resolver) {
        this._uriResolver = resolver;
    }
    
    @Override
    public DOM retrieveDocument(String baseURI, String href, Translet translet) {
        try {
            if (href.length() == 0) {
                href = baseURI;
            }
            Source resolvedSource = this._uriResolver.resolve(href, baseURI);
            if (resolvedSource == null) {
                StreamSource streamSource = new StreamSource(SystemIDResolver.getAbsoluteURI(href, baseURI));
                return this.getDOM(streamSource);
            }
            return this.getDOM(resolvedSource);
        }
        catch (TransformerException e) {
            if (this._errorListener != null) {
                this.postErrorToListener("File not found: " + e.getMessage());
            }
            return null;
        }
    }
    
    @Override
    public void error(TransformerException e) throws TransformerException {
        Throwable wrapped = e.getException();
        if (wrapped != null) {
            System.err.println(new ErrorMsg("ERROR_PLUS_WRAPPED_MSG", e.getMessageAndLocation(), wrapped.getMessage()));
        }
        else {
            System.err.println(new ErrorMsg("ERROR_MSG", e.getMessageAndLocation()));
        }
        throw e;
    }
    
    @Override
    public void fatalError(TransformerException e) throws TransformerException {
        Throwable wrapped = e.getException();
        if (wrapped != null) {
            System.err.println(new ErrorMsg("FATAL_ERR_PLUS_WRAPPED_MSG", e.getMessageAndLocation(), wrapped.getMessage()));
        }
        else {
            System.err.println(new ErrorMsg("FATAL_ERR_MSG", e.getMessageAndLocation()));
        }
        throw e;
    }
    
    @Override
    public void warning(TransformerException e) throws TransformerException {
        Throwable wrapped = e.getException();
        if (wrapped != null) {
            System.err.println(new ErrorMsg("WARNING_PLUS_WRAPPED_MSG", e.getMessageAndLocation(), wrapped.getMessage()));
        }
        else {
            System.err.println(new ErrorMsg("WARNING_MSG", e.getMessageAndLocation()));
        }
    }
    
    @Override
    public void reset() {
        this._method = null;
        this._encoding = null;
        this._sourceSystemId = null;
        this._errorListener = this;
        this._uriResolver = null;
        this._dom = null;
        this._parameters = null;
        this._indentNumber = 0;
        this.setOutputProperties(null);
        this._tohFactory = null;
        this._ostream = null;
    }
    
    static class MessageHandler extends com.sun.org.apache.xalan.internal.xsltc.runtime.MessageHandler
    {
        private ErrorListener _errorListener;
        
        public MessageHandler(ErrorListener errorListener) {
            this._errorListener = errorListener;
        }
        
        @Override
        public void displayMessage(String msg) {
            if (this._errorListener == null) {
                System.err.println(msg);
            }
            else {
                try {
                    this._errorListener.warning(new TransformerException(msg));
                }
                catch (TransformerException ex) {}
            }
        }
    }
}
